// Data/LateNoteData.H - This file is part of NotedELN

/* NotedELN is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   NotedELN is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NotedELN.  If not, see <http://www.gnu.org/licenses/>.
*/

// LateNoteData.H

#ifndef LATENOTEDATA_H

#define LATENOTEDATA_H

#include "GfxNoteData.h"

class LateNoteData: public GfxNoteData {
  Q_OBJECT;
public:
  LateNoteData(Data *parent=0);
  virtual ~LateNoteData();
  virtual void markModified(ModType mt=UserVisibleMod);
  /* This version of markModified never propagates, even for UserVisibleMod. */
  virtual Notebook *book() const;
  void setBook(Notebook *); // null means use parent's
  virtual class ResManager *resManager() const;
private:
  class Notebook *nb;
};

#endif
