body { 
  font-family: ubuntu, serif;
  font-size: 18px;
  background-color: white;
  color: #0000cc;
}

a:link { background-color: rgba(0, 130, 255, 0.15); color: inherit; 
         text-decoration: none }
a:visited { background-color: rgba(0, 130, 255, 0.15); color: inherit; 
         text-decoration: none }
a:hover {  background-color: rgba(0, 130, 255, 0.35); color: inherit; 
         text-decoration: none }

.footnoteref:link { background-color: white; color: #aa0088; text-decoration: none }
.footnoteref:visited { background-color: white; color: #aa0088; text-decoration: none }
.footnoteref:hover { background-color: white; color: red; text-decoration: none }

.title {
    font-size: 120%;
    color: #000088;
    margin-bottom: 25px;
}

.date {
    text-align: right;
    margin-top: 0px;
    margin-bottom: 7px;
    color: #000000;
}

.textblock {
    margin-top: 7px;
    margin-bottom: 7px;
    text-indent: 20px;
}

.gfxnote {
    color: #2255ff;
    font-size: 90%;
}

.latenote {
    color: #ee2200;
}

.tag {
    color: #aa0088;
}

.noindent {
    text-indent: 0px;
}
.dedent {
    text-indent: -20px;
    margin-left: 20px;
}
.displaytext {
    color: #000066;
    font-size: 90%;
    padding-left: 30px;
}

.footnote {
    margin-top: 7px;
    margin-left: 40px;
    text-indent: -40px;
}

.footnotes {
    margin-top: 30px;
    font-size: 90%;
}

.latenote {
    margin-top: 7px;
    margin-bottom: 7px;
}

.gfxnote {
    margin-top: 7px;
    margin-bottom: 7px;
}

.badurl {
    background-color: rgb(255,0,0,0.1);
}

.nores {
    background-color: rgb(255,0,0,0.1);
}

.emph {
    background-color: #ffff00;
}
